@@ -180,39 +180,98 @@ class Dynamics(BaseDynamics):
         if self._shape is None:
             self._shape = tmp_shape
 
+    @property
+    def shape(self):
+        """The shape of the states. """
+        return np.copy(self._shape)
 
-class DynamicsWithField(BaseDynamics):
+class MeanFieldDynamics(BaseAPIClass):
     """
-    Represents a specific time evolution of a density matrix together
-    with a coherent field.
+    Represents a time evolution of one or more system density matrices
+    and a coherent field comprising a MeanFieldSystem.
 
-    Parameters
+    parameters
     ----------
-    times: List[float] (default = None)
-        A list of points in time.
-    states: List[ndarray] (default = None)
-        A list of states at the times `times`.
-    fields: List[complex] (default = None)
-        A list of fields at the times `times`.
+    times: list[float] (default = None)
+        a list of points in time.
+    system_states_list: list[list[ndarray]] (default = None)
+        a list of lists such that the ith sublist is a list of states
+        for each system at time `times[i]`.
+    fields: list[complex] (default = None)
+        a list of fields at the times `times`.
     name: str
-        An optional name for the dynamics.
+        an optional name for the dynamics.
     description: str
-        An optional description of the dynamics.
+        an optional description of the dynamics.
     """
-    def __init__(
-            self,
-            times: Optional[List[float]] = None,
-            states: Optional[List[ndarray]] = None,
-            fields: Optional[List[complex]] = None,
-            name: Optional[Text] = None,
-            description: Optional[Text] = None) -> None:
-        """Create a DynamicsWithField object"""
+    def __init__(self,
+                 times: Optional[List[float]] = None,
+                 system_states_list: Optional[List[List[ndarray]]] = None,
+                 fields: Optional[List[complex]] = None,
+                 name: Optional[Text] = None,
+                 description: Optional[Text] = None) -> None:
+        """Create a MeanFieldDynamics object"""
         super().__init__(name, description)
+        self._times = []
         self._fields = []
-        times, states = _parse_times_states(times, states)
-        times, fields = _parse_times_fields(times, fields)
-        for time, state, field in zip(times, states, fields):
-            self.add(time, state, field)
+        self._system_dynamics = []
+        self._shapes = []
+        # check lengths of passed lists match
+        times, tmp_fields = _parse_times_fields(times, fields)
+        # Don't check for correct state shapes yet
+        times, tmp_states = _parse_times_states(times, system_states_list)
+        # Add each time, set of states and field (type checking in .add())
+        for time, states, field in zip(times, tmp_states, tmp_fields):
+            self.add(time, states, field)
+
+    def add(
+            self,
+            time: float,
+            system_states: List[ndarray],
+            field: complex) -> None:
+        """
+        Append a list of system states and field at a specific time
+        to the evolution.
+
+        Parameters
+        ----------
+        time: float
+            The point in time.
+        system_states: List[ndarray]
+            The state of each system at the time `time`.
+        field: complex
+            The field at the time `time`.
+        """
+        # Record time in mean-field system times list
+        tmp_time = _parse_time(time)
+        index = _find_list_index(self._times, tmp_time)
+        self._times.insert(index, tmp_time)
+        # Record field in mean-field system fields list
+        tmp_field = _parse_field(field)
+        self._fields.insert(index, tmp_field)
+        # Create list of Dynamics (one for each system), if not already done
+        if len(self._system_dynamics) == 0:
+            self._system_dynamics = [Dynamics(name="system {}".format(i))
+                                     for i in range(len(system_states))]
+            self._shape_list = [dynamics.shape for dynamics
+                                in self._system_dynamics]
+        else:
+            assert len(system_states) == len(self._system_dynamics),\
+                "Number of states to add ({}) does not match number of "\
+                "Dynamics objects in "\
+                "MeanFieldDynamics ({})".format(
+                        len(system_states), len(self._system_dynamics))
+        # Record state - state parsing done by Dynamics objects
+        for i, system_dynamics in enumerate(self._system_dynamics):
+            system_dynamics.add(time, system_states[i])
+
+    def __len__(self) -> int:
+        return len(self._times)
+
+    @property
+    def times(self) -> ndarray:
+        """Times of the dynamics. """
+        return np.array(self._times, dtype=NpDtypeReal)
 
     @property
     def fields(self) -> ndarray:
@@ -232,35 +291,13 @@ class DynamicsWithField(BaseDynamics):
         """
         if len(self) == 0:
             return None, None
-        return np.array(self._times, dtype=NpDtypeReal), np.array(self._fields,
-                dtype=NpDtype)
+        return np.array(self._times, dtype=NpDtypeReal),\
+                np.array(self._fields, dtype=NpDtype)
 
-    def add(
-            self,
-            time: float,
-            state: ndarray,
-            field: complex) -> None:
-        """
-        Append a state and field at a specific time to the time evolution.
-
-        Parameters
-        ----------
-        time: float
-            The point in time.
-        state: ndarray
-            The state at the time `time`.
-        field: complex
-            The field at the time `time`.
-        """
-        tmp_time = _parse_time(time)
-        tmp_state, tmp_shape = _parse_state(state, self._shape)
-        tmp_field = _parse_field(field)
-        index = _find_list_index(self._times, tmp_time)
-        self._times.insert(index, tmp_time)
-        self._states.insert(index, tmp_state)
-        self._fields.insert(index, tmp_field)
-        if self._shape is None:
-            self._shape = tmp_shape
+    @property
+    def system_dynamics(self) -> List[Dynamics]:
+        """Dynamics object for each system. """
+        return self._system_dynamics
 
 def _parse_times_states(times, states) -> Tuple[List[float],
         List[ndarray]] :
