@@ -23,7 +23,7 @@ framework and efficient characterization*, Phys. Rev. A 97, 012127 (2018).
 
 import os
 import tempfile
-from typing import Optional, Text
+from typing import Optional, Text, Union
 
 import numpy as np
 from numpy import ndarray
@@ -95,6 +95,11 @@ class BaseProcessTensor(BaseAPIClass):
         """ToDo. """
         return self._transform_in
 
+    @property
+    def max_step(self) -> Union[int, float]:
+        """ToDo"""
+        return len(self)
+
     @property
     def transform_out(self):
         """ToDo. """
@@ -181,11 +186,12 @@ class TrivialProcessTensor(BaseProcessTensor):
     """
     def __init__(
             self,
+            hilbert_space_dimension: Optional[int] = 1,
             name: Optional[Text] = None,
             description: Optional[Text] = None) -> None:
         """ToDo. """
         super().__init__(
-            hilbert_space_dimension=1,
+            hilbert_space_dimension=hilbert_space_dimension,
             name=name,
             description=description)
 
@@ -193,6 +199,10 @@ class TrivialProcessTensor(BaseProcessTensor):
         """Length of process tensor. """
         return 0
 
+    @property
+    def max_step(self) -> Union[int, float]:
+        return float('inf')
+
     def get_initial_tensor(self) -> ndarray:
         """ToDo. """
         return None
@@ -523,10 +533,10 @@ class FileProcessTensor(BaseProcessTensor):
 
         # transforms
         transform_in = np.array(self._f["transform_in"])
-        if transform_in == 0.0:
+        if np.allclose(transform_in, np.array([0.0])):
             transform_in = None
         transform_out = np.array(self._f["transform_out"])
-        if transform_out == 0.0:
+        if np.allclose(transform_out, np.array([0.0])):
             transform_out = None
 
         # initial tensor and mpo/cap/lam tensors
